/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.spez;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.junit.Assert;

import com.bitctrl.Constants;

import de.bsvrz.iav.gllib.gllib.dav.GanglinieMQ;
import de.bsvrz.iav.gllib.gllib.dav.GlProgAnfrage;
import de.bsvrz.iav.gllib.gllib.dav.Messwerte;
import de.bsvrz.iav.gllib.gllib.modell.onlinedaten.OdPrognoseGanglinienAnfrage;
import de.bsvrz.iav.gllib.gllib.modell.onlinedaten.OdPrognoseGanglinienAntwort;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.ganglinien.objekte.ApplikationGanglinienPrognose;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellglobal.objekte.Applikation;

/**
 * Stellvertreter für die Ganglinienprognose. Die Klasse dient zur Berei
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class GanglinienPrognoseProxy implements DatensatzUpdateListener {

	/**
	 * Konstruktor.
	 */
	public GanglinienPrognoseProxy() {

		final ApplikationGanglinienPrognose glProg = (ApplikationGanglinienPrognose) ObjektFactory.getInstanz()
				.getModellobjekt(ObjektFactory.getInstanz().getVerbindung().getLocalConfigurationAuthority());

		glProg.getOnlineDatensatz(OdPrognoseGanglinienAnfrage.class)
		.setSenke(OdPrognoseGanglinienAnfrage.Aspekte.Anfrage.getAspekt(), true);
		glProg.getOnlineDatensatz(OdPrognoseGanglinienAnfrage.class)
				.addUpdateListener(OdPrognoseGanglinienAnfrage.Aspekte.Anfrage.getAspekt(), this);
	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		try {
			final OdPrognoseGanglinienAnfrage.Daten daten = (OdPrognoseGanglinienAnfrage.Daten) event.getDatum();

			if (daten.isValid()) {

				final Applikation app = daten.getAbsender();
				final OdPrognoseGanglinienAntwort antwort = app.getOnlineDatensatz(OdPrognoseGanglinienAntwort.class);
				antwort.anmeldenSender(OdPrognoseGanglinienAntwort.Aspekte.Antwort.getAspekt());

				final OdPrognoseGanglinienAntwort.Daten antwortDaten = antwort.erzeugeDatum();
				antwortDaten.setAbsenderZeichen(daten.getAbsenderZeichen());

				final Iterator<GlProgAnfrage> iterator = daten.iterator();
				while (iterator.hasNext()) {
					final GlProgAnfrage anfrage = iterator.next();
					final GanglinieMQ ganglinie = new GanglinieMQ();
					ganglinie.setApproximationDaK(GanglinieMQ.APPROX_POLYLINE);
					ganglinie.setMessQuerschnitt(anfrage.getMessQuerschnitt());
					ganglinie.setPrognoseZeitraum(anfrage.getPrognoseZeitraum());
					ganglinie.putAll(getGanglinienDaten(anfrage.getPrognoseZeitraum().getStart()));
					antwortDaten.add(ganglinie);
				}

				antwort.sendeDaten(OdPrognoseGanglinienAntwort.Aspekte.Antwort.getAspekt(), antwortDaten);
			}

		} catch (final AnmeldeException e) {
			Assert.fail(e.getMessage());
		} catch (final DatensendeException e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * liefert die Daten der Ganglinie für den Testfall.
	 *
	 * @param anfangsZeit
	 *            der Anfangszeitpunkt
	 * @return die Liste der Daten
	 */
	private Map<? extends Long, ? extends Messwerte> getGanglinienDaten(final long anfangsZeit) {
		final Map<Long, Messwerte> result = new TreeMap<>();
		long start = anfangsZeit;
		final double[] werte = { 0, 0, 516, 811, 671, 374, 117, 361, 572, 721, 167, 347, 0, 0, 472, 769, 689, 329, 100,
				348, 530, 748, 281, 340 };

		for (final double wert : werte) {
			result.put(start, new Messwerte(wert, 0.0, 50.0, 50.0));
			start += 2 * Constants.MILLIS_PER_HOUR;
		}
		return result;
	}
}
